# This file was auto-generated by Fern from our API Definition.

import typing

WindowLoadUrlV1BodyWaitUntil = typing.Union[
    typing.Literal["load", "domContentLoaded", "complete", "noWait"], typing.Any
]
