# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ScrollToEdgeConfig(UniversalBaseModel):
    x_axis: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="xAxis")] = pydantic.Field(
        default=None
    )
    """
    Scroll to the left or right of the page.
    """

    y_axis: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="yAxis")] = pydantic.Field(
        default=None
    )
    """
    Scroll to the top or bottom of the page.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
