# This file was auto-generated by Fern from our API Definition.

from .types import CreateWindowInputV1BodyWaitUntil, WindowLoadUrlV1BodyWaitUntil

__all__ = ["CreateWindowInputV1BodyWaitUntil", "WindowLoadUrlV1BodyWaitUntil"]
