# This file was auto-generated by Fern from our API Definition.

import typing
from .proxy import Proxy
from .proxy_configuration_kind import ProxyConfigurationKind

SessionConfigV1Proxy = typing.Union[bool, Proxy, typing.List[ProxyConfigurationKind]]
