# This file was auto-generated by Fern from our API Definition.

import typing

VisualAnalysisConfigPartitionDirection = typing.Union[
    typing.Literal["vertical", "horizontal", "bidirectional"], typing.Any
]
