# This file was auto-generated by Fern from our API Definition.

import typing

StatusMessageStatus = typing.Union[
    typing.Literal["awaitingCapacity", "initializing", "running", "ended", "error", "disconnected"], typing.Any
]
