# This file was auto-generated by Fern from our API Definition.

import typing
from .airtop_proxy_configuration import AirtopProxyConfiguration
from .proxy_credentials import ProxyCredentials

Proxy = typing.Union[str, AirtopProxyConfiguration, ProxyCredentials]
