# This file was auto-generated by Fern from our API Definition.

import typing

BrowserWaitNavigationConfigWaitUntil = typing.Union[
    typing.Literal["load", "domcontentloaded", "networkidle0", "networkidle2"], typing.Any
]
